/*
 * DistancePointSegmentExample, calculate distance to line
 * Copyright (C) 2008 Pieter Iserbyt <pieter.iserbyt@gmail.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Example implementation of "Minimum Distance between a Point and a Line" as
 * described by Paul Bourke on
 * See http://local.wasp.uwa.edu.au/~pbourke/geometry/pointline/.
 */

//Modified to return projection to segment and distance to segment
//by Ioannis (John) Chatzikonstantinou in May 2009
// Also simplified
// http://prototy.blogspot.com

/**
 * Returns the projection of p3 to the segment defined by p1,p2;
 * 
 * @param p1
 *                First point of the segment
 * @param p2
 *                Second point of the segment
 * @param p3
 *                Point of which we want to know the projection on the segment
 *                defined by p1,p2
 * @return The projection of p3 to the segment defined by p1,p2
 */
public static Point2 pTS(Point2 p1, Point2 p2, Point2 p3) {

  final float xDelta = p2.x-p1.x;
  final float yDelta = p2.y-p1.y;
  final Point2 closestPoint;

  if ((xDelta == 0) && (yDelta == 0)) {
    closestPoint=p1;
  } 
  else {

    final float u = ((p3.x - p1.x)*xDelta + (p3.y - p1.y)*yDelta) / (xDelta*xDelta + yDelta*yDelta);

    if (u < 0) {
      closestPoint = p1;
    } 
    else if (u > 1) {
      closestPoint = p2;
    }
    else {
      closestPoint = new Point2(p1.x + u*xDelta, p1.y + u*yDelta);
    }
  }
  return(closestPoint);
}

/**
 * Returns the projection of p3 to the line defined by p1,p2;
 * 
 * @param p1
 *                First point of the segment
 * @param p2
 *                Second point of the segment
 * @param p3
 *                Point to which we want to know the distance of the line
 *                defined by p1,p2
 * @return The projection of p3 to the line defined by p1,p2
 */
public static Point2 pTL(Point2 p1, Point2 p2, Point2 p3) {

  final float xDelta = p2.x - p1.x;
  final float yDelta = p2.y - p1.y;
  final Point2 closestPoint;

  if ((xDelta == 0) && (yDelta == 0)) {
    closestPoint=new Point2(0,0);
  } 
  else {

    final float u = ((p3.x - p1.x) * xDelta + (p3.y - p1.y) * yDelta) / (xDelta * xDelta + yDelta * yDelta);

    closestPoint = new Point2(p1.x + u * xDelta, p1.y + u * yDelta);
  }
  return(closestPoint);
}
/**
 * Returns the distance of p3 to the segment defined by p1,p2;
 * 
 * @param p1
 *                First point of the segment
 * @param p2
 *                Second point of the segment
 * @param p3
 *                Point to which we want to know the distance of the segment
 *                defined by p1,p2
 * @return The distance of p3 to the segment defined by p1,p2
 */
public static float dTS(Point2 p1, Point2 p2, Point2 p3) {

  final Point2 closestPoint=pTS(p1, p2, p3);
  return closestPoint.distance(p3);
}
/**
 * Returns the distance of p3 to the line defined by p1,p2;
 * 
 * @param p1
 *                First point of the segment
 * @param p2
 *                Second point of the segment
 * @param p3
 *                Point to which we want to know the distance of the segment
 *                defined by p1,p2
 * @return The distance of p3 to the segment defined by p1,p2
 */
public static float dTL(Point2 p1, Point2 p2, Point2 p3) {

  final Point2 closestPoint=pTL(p1, p2, p3);
  return closestPoint.distance(p3);
}


